package com.perforce.hws.plugin.perforce.routes;

import com.perforce.hws.core.p4base.P4Methods;
import com.perforce.hws.core.p4base.ResultMap;
import com.perforce.hws.core.p4base.ResultMapMethods;
import com.perforce.hws.core.sessions.SessionData;
import com.perforce.hws.route.UserRoute;
import com.perforce.hws.utils.HWSSettings;
import com.perforce.hws.utils.ModelConversion;
import com.perforce.hws.utils.UsesServerHandles;

import static spark.Spark.halt;

/**
 * GET /p4/:api/users/:user
 */
public class PerforceGetUserRoute implements
		spark.Route,
		P4Methods,
		UserRoute,
		ModelConversion,
		ResultMapMethods,
		UsesServerHandles {

	@Override
	public Object handle(spark.Request request, spark.Response response) throws Exception {
		SessionData sessionData = request.attribute("sessionData");
		HWSSettings settings = request.attribute("settings");

		String server = request.params("server");
		String user = request.params("user");

		return withServerHandle(server, sessionData, settings, serverHandle -> {
			java.util.List<ResultMap> results = exec(serverHandle, "user", "-o", user);

			if (results.stream().anyMatch(this::isInvalidUserEmail)) {
				halt(404);
			}

			ResultMap resultMap = firstResultOrFail(results);

			java.util.function.Supplier<String> offsetSupplier = offsetSupplier(serverHandle);

			return toUserCommand(resultMap, offsetSupplier);
		});
	}
}
