package com.perforce.hws.plugin.perforce.routes;

import com.perforce.hws.core.p4base.P4Methods;
import com.perforce.hws.core.p4base.ResultMap;
import com.perforce.hws.core.sessions.SessionData;
import com.perforce.hws.utils.HWSSettings;
import com.perforce.hws.utils.ModelConversion;
import com.perforce.hws.utils.UsesServerHandles;
import spark.Request;
import spark.Response;
import spark.Route;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import static java.util.Collections.addAll;

/**
 * /api/p4/v16.1/{server}/fixes:
 */
public class PerforceListFixesRoute implements
		Route,
		P4Methods,
		ModelConversion,
		UsesServerHandles {

	@Override
	public Object handle(Request request, Response response) throws Exception {
		SessionData sessionData = request.attribute("sessionData");
		HWSSettings settings = request.attribute("settings");

		String server = request.params("server");
		String includeIntegrated = request.queryParams("includeIntegrated");
		String max = request.queryParams("max");
		String change = request.queryParams("change");
		String job = request.queryParams("job");
		String files = request.queryParams("files");

		List<String> args = new ArrayList<>();
		if (includeIntegrated != null && Boolean.valueOf(includeIntegrated)) {
			args.add("-i");
		}
		if (max != null) {
			addAll(args, "-m", max);
		}
		if (change != null) {
			addAll(args, "-c", change);
		}
		if (job != null) {
			addAll(args, "-j", job);
		}
		if (files != null) {
			args.add(files);
		}

		return withServerHandle(server, sessionData, settings, serverHandle -> {
			List<ResultMap> resultMapList = exec(serverHandle, "fixes", args.toArray(new String[0]));

			Supplier<String> offsetSupplier = offsetSupplier(serverHandle);

			return resultMapList.stream().map(m -> toFixesCommand(m, offsetSupplier))
					.collect(Collectors.toList());
		});
	}
}
