package com.perforce.hws.plugin.perforce.routes;

import com.perforce.hws.core.p4base.HasGson;
import com.perforce.hws.core.p4base.P4Methods;
import com.perforce.hws.core.p4base.ResultMap;
import com.perforce.hws.core.p4base.ResultMapMethods;
import com.perforce.hws.core.sessions.SessionData;
import com.perforce.hws.route.UserRoute;
import com.perforce.hws.route.UsesJsonRequests;
import com.perforce.hws.utils.HWSSettings;
import com.perforce.hws.utils.ModelConversion;
import com.perforce.hws.utils.UsesServerHandles;
import com.perforce.p4client.models.UserCommand;

import static spark.Spark.halt;

/**
 * PATCH /p4/:api/users/:user
 */
public class PerforceUpdateUserRoute implements
        spark.Route,
		P4Methods,
		ResultMapMethods,
		UserRoute,
		ModelConversion,
		HasGson,
		UsesJsonRequests,
		UsesServerHandles {

    @Override
    public Object handle(spark.Request request, spark.Response response) throws Exception {
        SessionData sessionData = request.attribute("sessionData");
        HWSSettings settings = request.attribute("settings");

        String server = request.params("server");
        String user = request.params("user");

        UserCommand updateCommand = readModel(request, UserCommand.class);
        java.util.Map<String, Object> updates = toRequestMap(updateCommand);

        return withServerHandle(server, sessionData, settings, serverHandle -> {

            java.util.List<ResultMap> results = exec(serverHandle, "user", "-o", user);
            ResultMap resultMap = firstResultOrFail(results);

            if (isInvalidUserEmail(resultMap)) {
                halt(404);
            }

            java.util.function.Supplier<String> offsetSupplier = offsetSupplier(serverHandle);

            UserCommand savedCommand = toUserCommand(resultMap, offsetSupplier);

            java.util.Map<String, Object> saved = toRequestMap(savedCommand);

            saved.putAll(updates);

            return toCommandResponse(execInput(serverHandle, saved, "user", "-f", "-i"));
        });
    }
}
